//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2011 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_ALGORITHM_UTF16_H_
#define _BUOLA_ALGORITHM_UTF16_H_

#include <buola/buola.h>
#include <unicode/utypes.h>

namespace buola {

///\addtogroup algorithm
///@{

///\name Character encoding
//@{

///UTF-16 string
typedef std::basic_string<UChar> TUString;

///converts a wide string to a UTF-16 string
inline TUString encode16(const std::wstring &pS)
{
    TUString lS;
    lS.resize(pS.size());

    for(int i=0;i<pS.size();i++)
    {
        /// \todo fix this and make it work for all characters
        if(pS[i]>0xffff)
            msg_warn() << "wide-character is not in the BMP when converting to UTF-16\n";
        lS[i]=pS[i];
    }

    return lS;
}

///converts a UTF-16 string to a wide string
inline std::wstring decode16(const TUString &pS)
{
    std::wstring lS;
    lS.resize(pS.size());

    for(int i=0;i<pS.size();i++)
        lS[i]=pS[i];

    return lS;
}

//@}

///@}

/*namespace buola*/ }

#endif
