//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2013 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_AUTO_CASTS_H_
#define _BUOLA_AUTO_CASTS_H_

#include <buola/auto/exceptions.h>

namespace buola {

///\addtogroup utility
///@{

///\name Casts
//@{
    
///semantic cast
///
///This cast operator provides simple conversions of values from one type to
///another based on their meaning. Its main use is to convert from arithmetic
///types to string types when it's not necessary to have a lot of control
///over the conversion.
///
///Conversions are defined for the basic integer and floating-type types to and from
///the default string types, but it is possible to define other conversions
///just by specializing the template.
template<typename tTo,typename tFrom>
tTo semantic_cast(const tFrom &pFrom);

///throwing cast
///
///This cast operator behaves exactly like dynamic_cast, except that it
///throws an exception if the cast is not possible. It should be used when
///you know that the cast will always work unless there is a programming
///error, as it is easier than checking the return of dynamic_cast, and
///it wouldn't make sense to continue running the program anyway
template<typename tTo,typename tFrom>
inline tTo throwing_cast(tFrom &&pSrc)
{
    if(tTo lReturn=dynamic_cast<tTo>(std::forward<tFrom>(pSrc)))
        return lReturn;

    throw XArgument("Invalid type in throwing_cast");
}

//@}

///@}

/*namespace buola*/ }

#endif
