//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2011 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_BUOLA_H_
#define _BUOLA_BUOLA_H_

#ifndef __GXX_EXPERIMENTAL_CXX0X__
#error "buola requires C++0x experimental support"
#endif

//the auto folder contains some files that should be included by every application using buola
//every file not in this folder should be included explicitly by applications using them
#include <buola/config.h>
#include <buola/auto/defines.h>
#include <buola/auto/exceptions.h>
#include <buola/auto/debug.h>
#include <buola/auto/intrusive_ptr.h>
#include <buola/auto/casts.h>
#include <buola/auto/forward.h>
#include <buola/auto/functions.h>
#include <buola/auto/memory.h>
#include <buola/auto/string.h>
#include <buola/auto/temp_ptr.h>
#include <buola/auto/traits.h>
#include <buola/auto/math.h>
#include <buola/auto/utility.h>

///\defgroup core core library
///\brief all classes and functions in this module are part of the core buola library which can be linked as -lbuolacore

///\defgroup algorithm algorithms
///\ingroup core
///\brief general purpose algorithms

///\defgroup application application management and event loop
///\ingroup core
///\brief this includes the functions that will usually be called from main for initializing the library, as well
///       as functions for event loop management

///\defgroup container containers and buffers
///\ingroup core
///\brief containers and buffers

///\defgroup datetime date/time
///\ingroup core
///\brief classes to manage date and time

///\defgroup debug debug functionality
///\ingroup core

///\defgroup exceptions exceptions
///\ingroup core
///\brief generic types for exceptions
///
///All these types inherit from std::exception

///\defgroup interfaces interfaces
///\ingroup core
///\brief interfaces are simple abstract classes

///\defgroup io input/output
///\ingroup core

///\defgroup math mathematical functions and constants
///\ingroup core

///\defgroup os operating system services
///\ingroup core
///\brief some functions and algorithms for string management

///\defgroup string string
///\ingroup core
///\brief some functions and algorithms for string management

///\defgroup types basic type definitions
///\ingroup core

///\defgroup utility utility
///\ingroup core
///\brief this contains classes and functions that don't fit in any other group, including memory management and
///       several templates.

#endif
