///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_CONTAINER_DETAIL_CBUFFERSTORAGE_H_
#define _BUOLA_CONTAINER_DETAIL_CBUFFERSTORAGE_H_

#include <buola/buola.h>
#include <buola/container/detail/callocatorasbase.h>

namespace buola { namespace detail {

template<typename tType,size_t vCapacity,typename tAlloc=std::allocator<tType> >
class CBufferStorage
{
    static_assert(std::is_trivial<tType>::value,"buffer types must be trivial");
    
public:
    typedef tType* TPointer;
    typedef tType& TReference;
    
    /// default constructor

    /// Initializes the storage. Since it is static, there's nothing to do here.
    CBufferStorage()
    {}

    /// returns the total current capacity of the buffer, in elements
    std::size_t GetCapacity()        {   return vCapacity;       }
    /// returns the total maximum capacity of the buffer, in elements
    std::size_t GetMaxCapacity()        {   return vCapacity;       }

    /// returns a pointer to the data
    TPointer GetStorage()    {   return mData;   }

private:
    tType mData[vCapacity];
};

template<typename tType,typename tAlloc>
class CBufferStorage<tType,cDynamic,tAlloc> : public CAllocatorAsBase<tAlloc>
{
    static_assert(std::is_trivial<tType>::value,"buffer types must be trivial");

public:
    typedef typename tAlloc::pointer TPointer;
    typedef typename tAlloc::reference TReference;
    
    using CAllocatorAsBase<tAlloc>::GetAllocator;
    
    /// constructor

    /// Initializes the storage. If the size is 0, no storage is allocated, and SetCapacity must be called
    /// to initialize it.
    CBufferStorage(std::size_t pCapacity=0)
        :   mCapacity(pCapacity)
    {
        if(mCapacity)
            mData=GetAllocator().allocate(mCapacity,nullptr);
        else
            mData=nullptr;
    }

    /// destructor
    ~CBufferStorage()
    {
        GetAllocator().deallocate(mData,mCapacity);
    }

    /// returns the total current capacity of the buffer, in elements
    std::size_t GetCapacity()            {   return mCapacity;       }
    /// returns the total maximum capacity of the buffer, in elements
    std::size_t GetMaxCapacity()         {   return (std::size_t)-1;       }

    /// sets the new capacity of the buffer

    /// Any data in the buffer is destroyed.
    void SetCapacity(std::size_t pCapacity)
    {
        if(mCapacity==pCapacity) return;

        if(mCapacity)
            GetAllocator().deallocate(mData,mCapacity);

        mCapacity=pCapacity;
        
        if(mCapacity)
            mData=GetAllocator().allocate(mCapacity,0);
        else
            mData=nullptr;
    }

    /// returns a pointer to the data
    TPointer GetStorage()    {   return mData;   }

private:
    TPointer mData;
    std::size_t mCapacity;
};

/*namespace detail*/ } /*namespace buola*/ }

#endif
