//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_DATETIME_CTIMEZONE_H_
#define _BUOLA_DATETIME_CTIMEZONE_H_

#include <buola/buola.h>

namespace buola {

///\addtogroup datetime
///@{

DECLARE_NAME(UTC);

////////////////////////////////////////////////////////////////////////////

/// time zone

/// Objects of this class represent a time zone.
///
/// It just contains a string representing an entry in the time zone database
/// of the system.
///
////////////////////////////////////////////////////////////////////////////

class CTimeZone
{
public:
    ///initializes the time zone to represent local time
    CTimeZone()
    {}

    ///initializes the time zone to represent UTC time
    CTimeZone(NUTC)
        :   mZone("UTC")
    {}

    ///initializes the time zone from a time zone name
    explicit CTimeZone(const std::string &pZone)
        :   mZone(pZone)
    {}

    ///returns true if the object represents the local time zone
    bool IsLocal() const    {   return mZone.empty();   }
    ///returns true if the object represents the local time zone
    bool IsUTC() const      {   return mZone=="UTC";    }

    ///returns the time zone name
    const std::string &GetName() const  {   return mZone;   }

private:
    std::string mZone;
};

///@}

/*namespace buola*/ }

#endif
