//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2013 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_FUNCTORS_PREDICATES_MATH_H_
#define _BUOLA_FUNCTORS_PREDICATES_MATH_H_

#include <buola/functors/predicates/unary.h>

namespace buola { namespace fn {

///\addtogroup functors
///@{

///\name Number classification predicates
//@{

/////////////////////////////////////////////////////////////////////////////////////////////////
///
/// isnan predicate
///
/// Evaluated with a single argument, returns true if the parameter is a NaN for its numeric type
///
/////////////////////////////////////////////////////////////////////////////////////////////////
struct AIsNaN : public GUnaryPredicate
{
    ///returns true if the character is of the given type
    template<typename tReal>
    bool operator()(tReal pReal) const
    {
        return is_nan(pReal);
    }
};

/////////////////////////////////////////////////////////////////////////////////////////////////
///
/// isfinite predicate
///
/// Evaluated with a single argument, returns true if the parameter is finite
///
/////////////////////////////////////////////////////////////////////////////////////////////////
struct AIsFinite : public GUnaryPredicate
{
    ///returns true if the character is of the given type
    template<typename tReal>
    bool operator()(tReal pReal) const
    {
        return is_finite(pReal);
    }
};

/////////////////////////////////////////////////////////////////////////////////////////////////
///
/// isinf predicate
///
/// Evaluated with a single argument, returns true if the parameter is infinite
///
/////////////////////////////////////////////////////////////////////////////////////////////////
struct AIsInf : public GUnaryPredicate
{
    ///returns true if the character is of the given type
    template<typename tReal>
    bool operator()(tReal pReal) const
    {
        return is_inf(pReal);
    }
};

//@}

///@}

/*namespace fn*/ } /*namespace buola*/ }

#endif
