///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CELEMENTBINARYOP_H_
#define _BUOLA_MAT_DETAIL_CELEMENTBINARYOP_H_

#include <buola/mat/cexpr.h>

namespace buola { namespace mat { namespace detail {

template<typename tLH,typename tRH,typename tOp>
struct MMatrixTraits<CElementBinaryOp<tLH,tRH,tOp>>
{
private:
    typedef MMatrixTraits<ODecay<tLH>> TLHTraits;
    typedef MMatrixTraits<ODecay<tRH>> TRHTraits;
    static_assert(TLHTraits::cRows==cUnknown||TRHTraits::cRows==cUnknown||
                  (TLHTraits::cRows==cDynamic)||//&&TRHTraits::cRows!=1)||
                  (TRHTraits::cRows==cDynamic)||//&&TLHTraits::cRows!=1)||
                  TLHTraits::cRows==TRHTraits::cRows,"wrong number of rows in binary operation");
    static_assert(TLHTraits::cCols==cUnknown||TRHTraits::cCols==cUnknown||
                  (TLHTraits::cCols==cDynamic)||//&&TRHTraits::cCols!=1)||
                  (TRHTraits::cCols==cDynamic)||//&&TLHTraits::cCols!=1)||
                  TLHTraits::cCols==TRHTraits::cCols,"wrong number of cols in binary operation");
    static_assert(std::is_same<typename TLHTraits::TScalar,typename TRHTraits::TScalar>::value,
                  "different scalar types in arguments to binary operator");
    
public:
    typedef typename TLHTraits::TScalar TScalar;
    static const size_t cRows=TLHTraits::cRows!=cDynamic?TLHTraits::cRows:
                                                                               TRHTraits::cRows;
    static const size_t cCols=TLHTraits::cCols!=cDynamic?TLHTraits::cCols:
                                                                               TRHTraits::cCols;
    static const bool cRowLinearAccess=TLHTraits::cRowLinearAccess&&TRHTraits::cRowLinearAccess;
    static const bool cColLinearAccess=TLHTraits::cColLinearAccess&&TRHTraits::cColLinearAccess;
    static const bool cRowPacketAccess=TLHTraits::cRowPacketAccess&&TRHTraits::cRowPacketAccess;
    static const bool cColPacketAccess=TLHTraits::cColPacketAccess&&TRHTraits::cColPacketAccess;
    static const bool cMightAlias=TLHTraits::cMightAlias||TRHTraits::cMightAlias;
    static const bool cElementAccess=TLHTraits::cElementAccess&&TRHTraits::cElementAccess;
    static const bool cEigenMappable=false;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=false;
    static const bool cIsMatrix=false;
};

//totally fixed size version
template<typename tLH,typename tRH,typename tOp>
class CElementBinaryOp : public CExpr<CElementBinaryOp<tLH,tRH,tOp>>
{
    typedef MMatrixTraits<CElementBinaryOp<tLH,tRH,tOp>> TSelfTraits;
    
public:
    CElementBinaryOp(tLH pLH,tRH pRH,tOp pOp)
        :   mLH(pLH)
        ,   mRH(pRH)
        ,   mOp(pOp)
    {
        assert(pLH.Rows()==pRH.Rows()&&pLH.Cols()==pRH.Cols());
    }

    CElementBinaryOp(const CElementBinaryOp&)=delete;
    CElementBinaryOp& operator=(const CElementBinaryOp&)=delete;

    size_t Rows() const  {   return mLH.Rows();  }
    size_t Cols() const  {   return mLH.Cols();  }
    size_t size() const  {   return mLH.size();  }

    template<typename tType>
    tType At(size_t r,size_t c) const
    {
        return mOp(mLH.template At<tType>(r,c),mRH.template At<tType>(r,c));
    }

    template<typename tType>
    tType At(size_t i) const
    {
        return mOp(mLH.template At<tType>(i),mRH.template At<tType>(i));
    }

    //left side can be element-accessed
    template<typename tDst,typename tLH2=tLH,typename tRH2=tRH>
    OEnableIf<QElementAccess<tLH2>,void> EvalTo(tDst &pDst) const
    {
        OMatrixType<tRH> lTmp(nSize,mRH.Rows(),mRH.Cols());
        
        mRH.EvalTo(lTmp);
        
        int lRows=pDst.Rows();
        int lCols=pDst.Cols();
        for(int r=0;r<lRows;r++)
            for(int c=0;c<lCols;c++)
                pDst.template At<OScalar<tDst>&>(r,c)=mOp(mLH.template At<OScalar<tDst>>(r,c),lTmp.template At<OScalar<tDst>>(r,c));
    }

    //right side can be element-accessed
    template<typename tDst,typename tLH2=tLH,typename tRH2=tRH>
    OEnableIf<QAnd<QElementAccess<tRH2>,QNot<QElementAccess<tLH2>>>,void> EvalTo(tDst &pDst) const
    {
        OMatrixType<tLH> lTmp(nSize,mLH.Rows(),mLH.Cols());

        mLH.EvalTo(lTmp);
        
        int lRows=pDst.Rows();
        int lCols=pDst.Cols();
        for(int r=0;r<lRows;r++)
            for(int c=0;c<lCols;c++)
                pDst.template At<OScalar<tDst>>(r,c)=mOp(lTmp.template At<OScalar<tDst>>(r,c),mRH.template At<OScalar<tDst>>(r,c));
    }
    
    //neither side can be element-accessed
    template<typename tDst,typename tLH2=tLH,typename tRH2=tRH>
    OEnableIf<QAnd<QNot<QElementAccess<tRH2>>,QNot<QElementAccess<tLH2>>>,void> EvalTo(tDst &pDst) const
    {
        OMatrixType<tLH> lTmpLH(nSize,mLH.Rows(),mLH.Cols());
        OMatrixType<tRH> lTmpRH(nSize,mRH.Rows(),mRH.Cols());

        mLH.EvalTo(lTmpLH);
        mRH.EvalTo(lTmpRH);
        
        int lRows=pDst.Rows();
        int lCols=pDst.Cols();
        for(int r=0;r<lRows;r++)
            for(int c=0;c<lCols;c++)
                pDst.template At<OScalar<tDst>&>(r,c)=mOp(lTmpLH.template At<OScalar<tDst>>(r,c),
                                                         lTmpRH.template At<OScalar<tDst>>(r,c));
    }
    
private:
    tLH mLH;
    tRH mRH;
    tOp mOp;
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
