///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CIDENTITYINITIALIZER_H_
#define _BUOLA_MAT_DETAIL_CIDENTITYINITIALIZER_H_

#include <buola/mat/cexpr.h>
#include <buola/mat/detail/csizeinfo.h>

namespace buola { namespace mat { namespace detail {

template<typename tScalar=UUnknown,size_t vRows=cDynamic,size_t vCols=cDynamic>
class CIdentityInitializer;

template<typename tScalar,size_t vRows,size_t vCols>
struct MMatrixTraits<CIdentityInitializer<tScalar,vRows,vCols>>
{
    typedef tScalar TScalar;
    static const size_t cRows=vRows;
    static const size_t cCols=vCols;
    static const bool cRowIdentityAccess=true;
    static const bool cColIdentityAccess=true;
    static const bool cRowPacketAccess=true;
    static const bool cColPacketAccess=true;
    static const bool cMightAlias=false;
    static const bool cElementAccess=true;
    static const bool cEigenMappable=false;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=false;
    static const bool cIsMatrix=false;
};

//totally fixed size version
template<typename tScalar,size_t vRows,size_t vCols>
class CIdentityInitializer : public CExpr<CIdentityInitializer<tScalar,vRows,vCols>>
                          , public CSizeInfo<vRows,vCols>
{
    typedef MMatrixTraits<CIdentityInitializer<tScalar,vRows>> TSelfTraits;
    typedef CSizeInfo<vRows,vCols> TSizeInfo;

public:
    CIdentityInitializer(size_t pRows,size_t pCols)
        :   TSizeInfo(pRows,pCols)
    {
    }

    CIdentityInitializer(const CIdentityInitializer&)=delete;
    CIdentityInitializer& operator=(const CIdentityInitializer&)=delete;

    using TSizeInfo::Rows;
    using TSizeInfo::Cols;

    template<typename tType>
    tType At(size_t r,size_t c) const
    {
        return (r==c)?tType(1.0):tType(0.0);
    }
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
