///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CINVERSE_H_
#define _BUOLA_MAT_DETAIL_CINVERSE_H_

#include <buola/mat/cexpr.h>
#include <buola/mat/detail/eigen_wrapper.h>

namespace buola { namespace mat { namespace detail {

template<typename tSub>
struct MMatrixTraits<CInverse<tSub>>
{
private:
    typedef MMatrixTraits<ODecay<tSub>> TSubTraits;

public:
    typedef typename TSubTraits::TScalar TScalar;
    static const size_t cRows=TSubTraits::cCols;
    static const size_t cCols=TSubTraits::cRows;
    static const bool cRowLinearAccess=false;
    static const bool cColLinearAccess=false;
    static const bool cRowPacketAccess=false;
    static const bool cColPacketAccess=false;
    static const bool cMightAlias=true;
    static const bool cElementAccess=false;
    static const bool cEigenMappable=false;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=false;
    static const bool cIsMatrix=false;
};

template<typename tSub>
class CInverse : public CExpr<CInverse<tSub>>
{
    typedef MMatrixTraits<CInverse<tSub>> TSelfTraits;

public:
    CInverse(tSub pSub)
        :   mSub(pSub)
    {}

    CInverse(const CInverse&)=delete;
    CInverse& operator=(const CInverse&)=delete;

    size_t Rows() const  {   return mSub.Cols(); }
    size_t Cols() const  {   return mSub.Rows(); }

    tSub Sub() const  {   return mSub;   }
    
    template<typename tMat>
    void EvalTo(tMat &pDst) const
    {
        ///\todo update this so that it only uses the LLT version when the matrix is actually self-adjoint
        ///and see why it doesn't improve performance
        eigen_map(pDst)=eigen_map(eigen_mappable(mSub)).inverse();
    }

private:
    tSub mSub;
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
