///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CLINEARINITIALIZER_H_
#define _BUOLA_MAT_DETAIL_CLINEARINITIALIZER_H_

#include <buola/mat/cexpr.h>
#include <buola/mat/detail/csizeinfo.h>

namespace buola { namespace mat { namespace detail {

template<typename tScalar=UUnknown,size_t vRows=cDynamic>
class CLinearInitializer;

template<typename tScalar,size_t vRows>
struct MMatrixTraits<CLinearInitializer<tScalar,vRows>>
{
    typedef tScalar TScalar;
    static const size_t cRows=vRows;
    static const size_t cCols=1;
    static const bool cRowLinearAccess=true;
    static const bool cColLinearAccess=true;
    static const bool cRowPacketAccess=true;
    static const bool cColPacketAccess=true;
    static const bool cMightAlias=false;
    static const bool cElementAccess=true;
    static const bool cEigenMappable=false;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=false;
    static const bool cIsMatrix=false;
};

//totally fixed size version
template<typename tScalar,size_t vRows>
class CLinearInitializer : public CExpr<CLinearInitializer<tScalar,vRows>>
                          , public CSizeInfo<vRows,1>
{
    typedef MMatrixTraits<CLinearInitializer<tScalar,vRows>> TSelfTraits;
    typedef CSizeInfo<vRows,1> TSizeInfo;

public:
    CLinearInitializer(size_t pRows,tScalar pMin,tScalar pStep)
        :   TSizeInfo(pRows,1)
        ,   mMin(pMin)
        ,   mStep(pStep)
    {
    }

    CLinearInitializer(const CLinearInitializer&)=delete;
    CLinearInitializer& operator=(const CLinearInitializer&)=delete;

    using TSizeInfo::Rows;
    using TSizeInfo::Cols;

    template<typename tType>
    tType At(size_t r,size_t /*c*/) const
    {
        return mMin+tScalar(r)*mStep;
    }

    template<typename tType>
    tType At(size_t i) const
    {
        return mMin+tScalar(i)*mStep;
    }

private:
    tScalar mMin;
    tScalar mStep;
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
