///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CPRODUCT_H_
#define _BUOLA_MAT_DETAIL_CPRODUCT_H_

#include <buola/mat/detail/eigen_wrapper.h>
#include <buola/mat/detail/cinverse.h>
#include <buola/mat/cmat.h>

namespace buola { namespace mat { namespace detail {

template<typename tLH,typename tRH>
struct MMatrixTraits<CProduct<tLH,tRH>>
{
private:
    typedef MMatrixTraits<ODecay<tLH>> TLHTraits;
    typedef MMatrixTraits<ODecay<tRH>> TRHTraits;
    
public:
    typedef typename TLHTraits::TScalar TScalar;
    static const size_t cRows=TLHTraits::cRows;
    static const size_t cCols=TRHTraits::cCols;
    static const bool cRowLinearAccess=false;
    static const bool cColLinearAccess=false;
    static const bool cRowPacketAccess=false;
    static const bool cColPacketAccess=false;
    static const bool cMightAlias=true;
    static const bool cElementAccess=false;
    static const bool cEigenMappable=false;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=false;
    static const bool cIsMatrix=false;
};

//totally fixed size version
template<typename tLH,typename tRH>
class CProduct : public CExpr<CProduct<tLH,tRH>>
{
    typedef MMatrixTraits<CProduct<tLH,tRH>> TSelfTraits;

public:
    CProduct(tLH pLH,tRH pRH)
        :   mLH(pLH)
        ,   mRH(pRH)
    {}

    CProduct(const CProduct&)=delete;
    CProduct& operator=(const CProduct&)=delete;

public:    
    size_t Rows() const  {   return mLH.Rows();  }
    size_t Cols() const  {   return mRH.Cols();  }

private:
    template<typename tDst,typename tOLH,typename tORH>
    void DoProduct(tDst &pDst,const tOLH &pLH,const tORH &pRH) const
    {
        eigen_map(pDst)=eigen_map(eigen_mappable(pLH))*eigen_map(eigen_mappable(pRH));
    }
    
    template<typename tDst,typename tOLH,typename tORH>
    void DoProduct(tDst &pDst,const CInverse<tOLH> &pLH,const tORH &pRH) const
    {
        eigen_map(pDst)=eigen_map(eigen_mappable(pLH.Sub())).colPivHouseholderQr().solve(eigen_map(eigen_mappable(pRH)));
    }

    template<typename tDst,typename tOLH,typename tORH>
    void DoProduct(tDst &pDst,const tORH &pLH,const CInverse<tOLH> &pRH) const
    {
        pDst.xxxxx();
        msg_warn() << "doing right-hand division\n";
        eigen_map(pDst)=eigen_map(eigen_mappable(pLH.Sub())).Transpose().colPivHouseholderQr().
                                solve(eigen_map(eigen_mappable(pRH)).Transpose()).Transpose();
    }

public:
    template<typename tMat>
    void EvalTo(tMat &pDst) const
    {
        DoProduct(pDst,mLH,mRH);
    }

private:
    tLH mLH;
    tRH mRH;
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
