///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CROWREDUCTION_H_
#define _BUOLA_MAT_DETAIL_CROWREDUCTION_H_

#include <buola/mat/cexpr.h>

namespace buola { namespace mat { namespace detail {

template<typename tSub,typename tReduction,typename tResult>
struct MMatrixTraits<CRowReduction<tSub,tReduction,tResult>>
{
private:
    typedef MMatrixTraits<ODecay<tSub>> TSubTraits;

public:
    typedef tResult TScalar;
    static const size_t cRows=TSubTraits::cRows;
    static const size_t cCols=1;
    static const bool cRowLinearAccess=false;
    static const bool cColLinearAccess=false;
    static const bool cRowPacketAccess=false;
    static const bool cColPacketAccess=false;
    static const bool cMightAlias=true;
    static const bool cElementAccess=TSubTraits::cElementAccess;
    static const bool cEigenMappable=false;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=false;
    static const bool cIsMatrix=false;
};

template<typename tSub,typename tReduction,typename tResult=OScalar<tSub>>
class CRowReduction : public CExpr<CRowReduction<tSub,tReduction,tResult>>
{
    typedef MMatrixTraits<CRowReduction<tSub,tReduction,tResult>> TSelfTraits;

public:
    CRowReduction(tSub pSub,const tReduction &pReduction)
        :   mSub(pSub)
        ,   mReduction(pReduction)
    {}

    CRowReduction(const CRowReduction&)=delete;
    CRowReduction& operator=(const CRowReduction&)=delete;

    size_t Rows() const  {   return mSub.Rows(); }
    size_t Cols() const  {   return 1; }

    template<typename tType>
    tType At(size_t r,size_t c) const       
    {   
        return mReduction(mSub(r,nAll));
    }
    
    template<typename tType>
    tType At(size_t i) const
    {   
        return mReduction(mSub(i,nAll));
    }

    template<typename tDst>
    void EvalTo(tDst &pDst) const
    {
        const auto &lSub=element_accessible(mSub);
        
        int lRows=pDst.Rows();
        for(int r=0;r<lRows;r++)
            pDst.template At<OScalar<tDst>>(r,0)=mReduction(lSub(r,nAll));
    }
    
private:
    tSub mSub;
    tReduction mReduction;
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
