///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CROWSELECTION_H_
#define _BUOLA_MAT_DETAIL_CROWSELECTION_H_

#include <buola/mat/cexpr.h>

namespace buola { namespace mat { namespace detail {

template<typename tSub,typename tIndices>
struct MMatrixTraits<CRowSelection<tSub,tIndices>>
{
private:
    typedef MMatrixTraits<ODecay<tSub>> TSubTraits;

public:
    typedef typename TSubTraits::TScalar TScalar;
    
    static const size_t cRows=cDynamic;
    static const size_t cCols=TSubTraits::cCols;
    static const bool cRowLinearAccess=false;
    static const bool cColLinearAccess=false;
    static const bool cRowPacketAccess=false;
    static const bool cColPacketAccess=false;
    static const bool cMightAlias=true;
    static const bool cElementAccess=TSubTraits::cElementAccess;
    static const bool cEigenMappable=false;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=false;
    static const bool cIsMatrix=false;
};

//totally fixed size version (no full rows or columns)
template<typename tSub,typename tIndices>
class CRowSelection : public CExpr<CRowSelection<tSub,tIndices>>
{
    typedef MMatrixTraits<CRowSelection<tSub,tIndices>> TSelfTraits;

public:
    template<typename tOIndices>
    CRowSelection(tSub pSub,tOIndices&& pIndices)
        :   mSub(pSub)
        ,   mIndices(std::forward<tOIndices>(pIndices))
    {}

    CRowSelection(const CRowSelection&)=delete;
    CRowSelection& operator=(const CRowSelection&)=delete;

    size_t Rows() const
    {
        return mIndices.size();
    }
    
    size_t Cols() const
    {
        return mSub.Cols();
    }
    
    size_t size()
    {
        return Rows()*Cols();
    }
    
    template<typename tType> tType At(size_t r,size_t c) const
    {   
        return mSub.template At<tType>(mIndices[r],c);    
    }

    template<typename tDst>
    void EvalTo(tDst &pDst) const
    {
        ///\todo consider posible aliasing, and whether it should be solved here or at an outer level
        const auto &lSub=element_accessible(mSub);
        
        int lRows=pDst.Rows();
        int lCols=pDst.Cols();
        
        for(int r=0;r<lRows;r++)
            for(int c=0;c<lCols;c++)
                pDst.template At<OScalar<tDst>>(r,c)=lSub.template At<OScalar<tDst>>(mIndices[r],c);
    }

private:
    tSub mSub;
    tIndices mIndices;
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
