///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CTRANSPOSE_H_
#define _BUOLA_MAT_DETAIL_CTRANSPOSE_H_

#include <buola/mat/cexpr.h>

namespace buola { namespace mat { namespace detail {

template<typename tSub>
struct MMatrixTraits<CTranspose<tSub>>
{
private:
    typedef MMatrixTraits<ODecay<tSub>> TSubTraits;

public:
    typedef typename TSubTraits::TScalar TScalar;
    static const size_t cRows=TSubTraits::cCols;
    static const size_t cCols=TSubTraits::cRows;
    static const bool cRowLinearAccess=TSubTraits::cColLinearAccess;
    static const bool cColLinearAccess=TSubTraits::cRowLinearAccess;
    static const bool cRowPacketAccess=TSubTraits::cColPacketAccess;
    static const bool cColPacketAccess=TSubTraits::cRowPacketAccess;
    static const bool cMightAlias=true;
    static const bool cElementAccess=TSubTraits::cElementAccess;
    static const bool cEigenMappable=TSubTraits::cEigenMappable;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=detail::QAssignable<tSub>::value;
    static const bool cIsMatrix=false;
};

template<typename tSub>
class CTranspose : public CExpr<CTranspose<tSub>>
{
    typedef MMatrixTraits<CTranspose<tSub>> TSelfTraits;

public:
    CTranspose(tSub pSub)
        :   mSub(pSub)
    {}

    CTranspose(const CTranspose&)=delete;
    CTranspose& operator=(const CTranspose&)=delete;

    size_t Rows() const  {   return mSub.Cols(); }
    size_t Cols() const  {   return mSub.Rows(); }

    template<typename tType>
    tType At(size_t r,size_t c) const       {   return mSub.template At<tType>(c,r);    }
    template<typename tType>
    tType At(size_t i) const                {   return mSub.template At<tType>(i);    }

    typename TSelfTraits::TScalar *Data()           {   return mSub.Data(); }
    const typename TSelfTraits::TScalar *Data() const     {   return mSub.Data(); }
    
    template<typename tDst>
    void EvalTo(tDst &pDst) const
    {
        ///\todo make it more efficient by calling eval on the transposed version of pDst, whenever that works
        const auto &lSub=element_accessible(mSub);
        
        int lRows=pDst.Rows();
        int lCols=pDst.Cols();
        
        for(int r=0;r<lRows;r++)
            for(int c=0;c<lCols;c++)
                pDst.template At<OScalar<tDst>>(r,c)=lSub.template At<OScalar<tDst>>(c,r);
    }

private:
    tSub mSub;
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
