///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_DETAIL_CVJOIN_H_
#define _BUOLA_MAT_DETAIL_CVJOIN_H_

#include <buola/mat/cexpr.h>

namespace buola { namespace mat { namespace detail {

template<typename tLH,typename tRH>
struct MMatrixTraits<CVJoin<tLH,tRH>>
{
private:
    typedef MMatrixTraits<ODecay<tLH>> TLHTraits;
    typedef MMatrixTraits<ODecay<tRH>> TRHTraits;

    static_assert(std::is_same<typename TLHTraits::TScalar,typename TRHTraits::TScalar>::value,
                  "different scalar types in arguments to binary operator");

public:
    typedef typename TLHTraits::TScalar TScalar;
    
    ///\todo do it better!!!
    static const size_t cRows=cDynamic;
    static const size_t cCols=cDynamic;
    static const bool cRowLinearAccess=false;
    static const bool cColLinearAccess=false;
    static const bool cRowPacketAccess=false;
    static const bool cColPacketAccess=false;
    static const bool cMightAlias=true;
    static const bool cElementAccess=false;
    static const bool cEigenMappable=false;
    static const bool cKeepConstness=false;
    static const bool cExprAssignable=false;
    static const bool cIsMatrix=false;
};

//totally fixed size version (no full rows or columns)
template<typename tLH,typename tRH>
class CVJoin : public CExpr<CVJoin<tLH,tRH>>
{
    typedef MMatrixTraits<CVJoin<tLH,tRH>> TSelfTraits;

public:
    CVJoin(tLH pLH,tRH pRH)
        :   mLH(pLH)
        ,   mRH(pRH)
    {}

    CVJoin(const CVJoin&)=delete;
    CVJoin& operator=(const CVJoin&)=delete;

    size_t Rows() const
    {
        return mLH.Rows()+mRH.Rows();
    }
    
    size_t Cols() const
    {
        return mLH.Cols();
    }
    
    size_t size()
    {
        return Rows()*Cols();
    }
    
    template<typename tDst>
    void EvalTo(tDst &pDst) const
    {
        ///\todo make it more efficient by using blocks, so that sub-joins need not be evaluated into a temporary
        const auto &lLH=element_accessible(mLH);
        const auto &lRH=element_accessible(mRH);
        
        int lRowsLH=lLH.Rows();
        int lRowsRH=lRH.Rows();
        int lCols=pDst.Cols();
        
        for(int r=0;r<lRowsLH;r++)
        {
            for(int c=0;c<lCols;c++)
                pDst.template At<OScalar<tDst>>(r,c)=lLH.template At<OScalar<tDst>>(r,c);
        }
        for(int r=0;r<lRowsRH;r++)
        {
            for(int c=0;c<lCols;c++)
                pDst.template At<OScalar<tDst>>(r+lRowsLH,c)=lRH.template At<OScalar<tDst>>(r,c);
        }
    }

private:
    tLH mLH;
    tRH mRH;
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
