///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_MAT_REDUCTIONS_H_
#define _BUOLA_MAT_REDUCTIONS_H_

#include <buola/mat/cexpr.h>
#include <buola/mat/detail/reductions.h>

namespace buola { namespace mat {

template<typename tDerived>
detail::CColwiseOp<const tDerived&> cols(const CExpr<tDerived> &pExpr)
{
    return {pExpr.Derived()};
}
    
template<typename tDerived>
detail::CRowwiseOp<const tDerived&> rows(const CExpr<tDerived> &pExpr)
{
    return {pExpr.Derived()};
}
    
template<typename tExpr>
detail::OScalar<tExpr> sum(const CExpr<tExpr> &pE)
{
    return detail::CReductionSum()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionSum> sum(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionSum> sum(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
detail::OScalar<tExpr> prod(const CExpr<tExpr> &pE)
{
    return detail::CReductionProd()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionProd> prod(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionProd> prod(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
detail::OScalar<tExpr> mean(const CExpr<tExpr> &pE)
{
    return detail::CReductionMean()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionMean> mean(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionMean> mean(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
detail::OScalar<tExpr> norm(const CExpr<tExpr> &pE)
{
    return detail::CReductionNorm()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionNorm> norm(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionNorm> norm(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
detail::OScalar<tExpr> norm2(const CExpr<tExpr> &pE)
{
    return detail::CReductionNorm2()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionNorm2> norm2(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionNorm2> norm2(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
detail::OScalar<tExpr> min(const CExpr<tExpr> &pE)
{
    return detail::CReductionMin()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionMin> min(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionMin> min(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
bool any(const CExpr<tExpr> &pE)
{
    return detail::CReductionAny()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionAny,bool> any(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionAny,bool> any(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
bool all(const CExpr<tExpr> &pE)
{
    return detail::CReductionAll()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionAll,bool> all(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionAll,bool> all(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
size_t count(const CExpr<tExpr> &pE)
{
    return detail::CReductionCount()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionCount,size_t> count(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionCount,size_t> count(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
detail::OScalar<tExpr> max(const CExpr<tExpr> &pE)
{
    return detail::CReductionMax()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionMax> max(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionMax> max(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
detail::OScalar<tExpr> var(const CExpr<tExpr> &pE)
{
    return detail::CReductionVar()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionVar> var(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionVar> var(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tExpr>
detail::OScalar<tExpr> stddev(const CExpr<tExpr> &pE)
{
    return detail::CReductionStdDev()(detail::element_accessible(pE.Derived()));
}

template<typename tSub>
detail::CColReduction<const tSub&,detail::CReductionStdDev> stddev(const detail::CColwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tSub>
detail::CRowReduction<const tSub&,detail::CReductionStdDev> stddev(const detail::CRowwiseOp<tSub> &pE)
{
    return {pE.Sub(),{}};
}

template<typename tDerived,typename tPredicate>
detail::OScalar<tDerived> sum_if(const CExpr<tDerived> &pE,tPredicate &&pPred)
{
    ///\todo optimize this
    const auto &lE=detail::element_accessible(pE.Derived());
    
    detail::OScalar<tDerived> lSum=0.0;
    
    for(int r=0;r<lE.Rows();r++)
    {
        for(int c=0;c<lE.Cols();c++)
        {
            if(pPred(lE(r,c)))
                lSum+=lE(r,c);
        }
    }    
    
    return lSum;
}

template<typename tDerived>
detail::OScalar<tDerived> trace(const CExpr<tDerived> &pE)
{
    const auto &lE=detail::element_accessible(pE.Derived());
    int lDim=std::min(lE.Rows(),lE.Cols());

    detail::OScalar<tDerived> lSum=0.0;
    
    for(int i=0;i<lDim;i++)
        lSum+=lE(i,i);

    return lSum;
}

template<typename tDerived>
std::pair<std::size_t,std::size_t> indmax(const CExpr<tDerived> &pE)
{
    const auto &lE=detail::element_accessible(pE.Derived());
    
    detail::OScalar<tDerived> lMax=-INFINITY;
    std::pair<std::size_t,std::size_t> lIndMax(-1,-1);
    
    for(int r=0;r<lE.Rows();r++)
    {
        for(int c=0;c<lE.Cols();c++)
        {
            if(lE(r,c)>lMax)
            {
                lMax=lE(r,c);
                lIndMax={r,c};
            }
        }
    }
    
    return lIndMax;
}

template<typename tLH,typename tRH>
detail::OScalar<tLH> distance(const CExpr<tLH> &pLH,const CExpr<tRH> &pRH)
{
    return norm(pRH-pLH);
}

template<typename tLH,typename tRH>
detail::OScalar<tLH> distance2(const CExpr<tLH> &pLH,const CExpr<tRH> &pRH)
{
    return norm2(pRH-pLH);
}

template<typename tDerived>
detail::OMatrixType<tDerived> normalize(const CExpr<tDerived> &pE)
{
    return pE/norm(pE);
}

template<typename tDerived,typename tPredicate>
size_t count_if(const CExpr<tDerived> &pE,tPredicate &&pPred)
{
    const auto &lE=detail::element_accessible(pE.Derived());
    
    size_t lCount=0;
    
    for(int r=0;r<lE.Rows();r++)
    {
        for(int c=0;c<lE.Cols();c++)
        {
            if(pPred(lE(r,c)))
                lCount++;
        }
    }    
    
    return lCount;
}

/*namespace mat*/ } /*namespace buola*/ }

#endif
