//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2011 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_UTILITY_OPERATORS_H_
#define _BUOLA_UTILITY_OPERATORS_H_

#include <buola/buola.h>

namespace buola { namespace ops {

///\addtogroup utility
///@{

/////////////////////////////////////////////////////////////////////////////////////////////////

/// comparison operators

/// Inherit from this class to define comparison operators from operator< and operator==
///
/////////////////////////////////////////////////////////////////////////////////////////////////

template<typename tType>
struct UComparison
{
    ///returns true if \p pLH <= \p pRH
    friend bool operator<=(const tType &pLH,const tType &pRH)
    {
        return !(pRH<pLH);
    }

    ///returns true if \p pLH >= \p pRH
    friend bool operator>=(const tType &pLH,const tType &pRH)
    {
        return !(pLH<pRH);
    }
    
    ///returns true if \p pLH > \p pRH
    friend bool operator>(const tType &pLH,const tType &pRH)
    {
        return pRH<pLH;
    }

    ///returns true if \p pLH != \p pRH
    friend bool operator!=(const tType &pLH,const tType &pRH)
    {
        return !(pLH==pRH);
    }
};

/////////////////////////////////////////////////////////////////////////////////////////////////

/// arithmetic operators

/// Inherit from this class to define arithmetic operators from the assignment arithmetic operators
///
/////////////////////////////////////////////////////////////////////////////////////////////////

template<typename tType>
struct UArithmetic
{
    ///returns \p pLH + \p pRH
    friend tType operator+(const tType &pLH,const tType &pRH)
    {
        tType lResult(pLH);
        return lResult+=pRH;
    }
};

///@}

/*namespace ops*/ } /*namespace buola*/ }

#endif
