///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_UTILITY_UCONNECTION_H_
#define _BUOLA_UTILITY_UCONNECTION_H_

#include <buola/buola.h>
#include <functional>
#include <set>

namespace buola {

template<typename tSig>
class UConnection
{
    typedef std::function<tSig> tFunction;
public:
    USignal()
    {}
    
    ~USignal()
    {
        for(auto i=mFunctions.begin();i!=mFunctions.end();i++)
            delete *i;
    }
    
    template<typename tFunc>
    tFunction *Connect(tFunc &&pFunc)
    {
        tFunction *lFunc=new tFunction(std::forward<tFunc>(pFunc));
        
        mFunctions.push_back(lFunc);
        return lFunc;
    }
    
    std::size_t Count()  {   return mFunctions.size();   }
    
    template<typename... tArgs>
    void operator()(tArgs&&... pArgs)
    {
        bool lErased=false;
        for(auto i=mFunctions.begin();i!=mFunctions.end();++i)
        {
            tFunction *lF=*i;
            if(*lF)
                (*lF)(std::forward<tArgs>(pArgs)...);
            else
            {
                *i=nullptr;
                lErased=true;
            }
        }
        if(lErased)
        {
            auto lEraseFirst=std::remove(mFunctions.begin(),mFunctions.end(),(tFunction*)nullptr);
            for(auto i=lEraseFirst;i<mFunctions.end();i++)
                delete *i;
            mFunctions.resize(lEraseFirst-mFunctions.begin());
        }
    }
    
private:
    std::vector<tFunction*> mFunctions;
};
    
/*namespace buola*/ }

#endif
