///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_UTILITY_USTORAGE_H_
#define _BUOLA_UTILITY_USTORAGE_H_

#include <buola/buola.h>

namespace buola { namespace mat { namespace detail {

//class for fixed/dynamic storage with move semantics and 0 specialization
//it doesn't hold the size for the dynamic version

//fixed size version
template<typename tScalar,size_t vSize>
class UStorage
{
public:
    UStorage() {}

    UStorage(const size_t pSize)
    {
        assert(pSize==vSize);
    }

    UStorage(const UStorage&)=default;

    UStorage(const UStorage &pO,size_t pSize)
    {
        assert(pSize==vSize);
        copy_array(pO.Data(),mData,vSize);
    }

    UStorage(const UStorage<tScalar,cDynamic> &pO,size_t pSize=vSize)
    {
        assert(pSize==vSize);
        copy_array(pO.Data(),mData,vSize);
    }

    UStorage &operator=(const UStorage&)=delete;

    void Assign(const UStorage &pO,size_t pSize=vSize)
    {
        assert(pSize==vSize);
        copy_array(pO.Data(),mData,vSize);
    }

    void Assign(const UStorage<tScalar,cDynamic> &pO,size_t pSize=vSize)
    {
        assert(pSize==vSize);
        copy_array(pO.Data(),mData,vSize);
    }

    tScalar *Data()             {   return mData;   }
    const tScalar *Data() const {   return mData;   }

    tScalar &At(size_t i)       {   return mData[i];}
    tScalar At(size_t i) const  {   return mData[i];}
    
    //doesn't preserve any data
    void Resize(size_t pSize)
    {
        assert(pSize==vSize);
    }

private:
    tScalar mData[vSize];
};

//dynamic size version
template<typename tScalar>
class UStorage<tScalar,cDynamic>
{
public:
    UStorage()
        :   mData(nullptr)
    {}

    explicit UStorage(size_t pSize)
        :   mData(new tScalar[pSize])
    {}

    UStorage(const UStorage&)=delete;

    template<size_t vSize>
    UStorage(const UStorage<tScalar,vSize> &pO,size_t pSize)
        :   mData(new tScalar[pSize])
    {
        copy_array(pO.Data(),mData,pSize);
    }

    UStorage(UStorage &&pO)
        :   mData(pO.mData)
    {
        pO.mData=nullptr;
    }

    ~UStorage()
    {
        delete[] mData;
    }

    UStorage &operator=(const UStorage&)=delete;

    template<size_t vSize>
    void Assign(const UStorage<tScalar,vSize> &pO,size_t pSize)
    {
        delete[] mData;
        mData=new tScalar[pSize];
        copy_array(pO.Data(),mData,pSize);
    }

    void Assign(UStorage &&pO)
    {
        using std::swap;
        swap(mData,pO.mData);
    }

    //doesn't preserve any data
    void Resize(size_t pSize)
    {
        delete[] mData;
        mData=new tScalar[pSize];
    }

    tScalar *Data()             {   return mData;   }
    const tScalar *Data() const {   return mData;   }

    tScalar &At(size_t i)       {   return mData[i];}
    tScalar At(size_t i) const  {   return mData[i];}

private:
    tScalar *mData;
};

//zero size version
//it has the dynamic interface to be able to use with the 0*cDynamic size version
template<typename tScalar>
class UStorage<tScalar,0>
{
public:
    UStorage() {}

    UStorage(size_t pSize)
    {
        assert(pSize==0);
    }

    UStorage(const UStorage&)=default;

    UStorage(const UStorage<tScalar,cDynamic>&,size_t pSize)
    {
        assert(pSize==0);
    }

    void Assign(const UStorage&)
    {}

    void Assign(const UStorage<tScalar,cDynamic>&,size_t pSize)
    {
        assert(pSize==0);
    }

    void Resize(size_t pSize)
    {
        assert(pSize==0);
    }

    tScalar *Data()             {   return nullptr;     }
    const tScalar *Data() const {   return nullptr;     }

    tScalar &At(size_t)       {   assert(false);      }
    tScalar At(size_t) const  {   assert(false);      }
};

/*namespace detail*/ } /*namespace mat*/ } /*namespace buola*/ }

#endif
