///\todo upgrade

//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//  This file is part of the buola project (https://code.google.com/p/buola/).  //
//                                                                              //
//  Copyright(c) 2007-2012 Xavi Gratal                                          //
//  gratal AT gmail DOT com                                                     //
//                                                                              //
//  Buola is free software: you can redistribute it and/or modify               //
//  it under the terms of the GNU General Public License as published by        //
//  the Free Software Foundation, either version 3 of the License, or           //
//  (at your option) any later version.                                         //
//                                                                              //
//  Buola is distributed in the hope that it will be useful,                    //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of              //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               //
//  GNU General Public License for more details.                                //
//                                                                              //
//  You should have received a copy of the GNU General Public License           //
//  along with buola.  If not, see <http://www.gnu.org/licenses/>.              //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

#ifndef _BUOLA_ZZ_ZPLUSOP_H_
#define _BUOLA_ZZ_ZPLUSOP_H_

#include <buola/zz/zbase.h>
#include <buola/zz/detail/vector.h>
#include <buola/zz/utility.h>

namespace buola { namespace zz {

template<typename tA>
class ZPlusOp : public ZBase<ZPlusOp<tA>>
{
public:
    template<typename tIt>
    struct tAttribute
    {
        typedef typename detail::MVectorType<typename tA::template tAttribute<tIt>::tType>::type tType;
    };
    
    ZPlusOp(const ZPlusOp &pRH)
        :   mA(pRH.mA)
    {}
    
    ZPlusOp(ZPlusOp &pRH)
        :   mA(pRH.mA)
    {}
    
    ZPlusOp(ZPlusOp &&pRH)
        :   mA(std::move(pRH.mA))
    {}
    
    template<typename tPA>
    explicit ZPlusOp(tPA &&pA)
        :   mA(std::forward<tPA>(pA))
    {}
    
    template<typename tIt,typename tAttr,typename tCtx,typename tSk>
    bool Parse(tIt &pB,const tIt &pE,tAttr &pAttr,tCtx &pCtx,const tSk &pSk) const
    {
        detail::clear_container(pAttr);
        
        {
            typename tA::template tAttribute<tIt>::tType lValue;
            if(!mA.Parse(pB,pE,lValue,pCtx,pSk))
                return false;
            detail::push_back_into(pAttr,std::move(lValue));
        }
        
        while(true)
        {
            typename tA::template tAttribute<tIt>::tType lValue;
            if(!mA.Parse(pB,pE,lValue,pCtx,pSk))
                return true;
            detail::push_back_into(pAttr,std::move(lValue));
        }
    }

    template<typename tIt,typename tAttr,typename tCtx,typename tSk>
    bool Parse(tIt &pB,const tIt &pE,UUnused &pAttr,tCtx &pCtx,const tSk &pSk) const
    {
        {
            if(!mA.Parse(pB,pE,pAttr,pCtx,pSk))
                return false;
        }
        
        while(true)
        {
            if(!mA.Parse(pB,pE,pAttr,pCtx,pSk))
                return true;
        }
    }
private:    
    tA mA;
};

/*namespace zz*/ } /*namespace buola*/ }

#endif
